/* SIMulador Sistema PLAnetario, M.Alvarez 2018
 * Trabajo para el TFM
 * del Master de Astrofísica de la UAM
 * 
 * Copyright (c) 2018 Mariano Alvarez Fernandez (malfer@telefonica.net)
 *
 * This file is part of simspla
 *
 * simspla is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * simspla is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with rebound.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "simspla.h"

static void imprime_body(Body *b)
{
    printf("Body %s %d %g ", b->nombre, b->tipo, b->masa);
    printf("%g %g %g ", b->pos.x, b->pos.y, b->pos.z);
    printf("%g %g %g\n", b->v.x, b->v.y, b->v.z);
}

void imprime_system(System *s)
{
    int i;
  
    printf("%s %d %d %f %ld %ld\n", s->nombre, s->nbodies, s->nbcforce,
           s->stepdt, s->nsteps, s->ncycles);
  
    for (i=0; i<s->nbodies; i++) {
        printf("%2d ", i);
        imprime_body(&(s->astro[i]));
    }
    printf("---\n");
}

static void guarda_body_reg_txt(FILE *f, Body *b, long reg, long first, double incrt)
{
    fprintf(f, "%.0f ", (reg+first)*incrt);
    fprintf(f, "%.15e %.15e %.15e ", b->reg[reg].pos.x, b->reg[reg].pos.y, b->reg[reg].pos.z);
    fprintf(f, "%.15e %.15e %.15e ", b->reg[reg].v.x, b->reg[reg].v.y, b->reg[reg].v.z);
    fprintf(f, "%.15e %.15e\n", b->reg[reg].hillr, b->reg[reg].distm);
}

static void guarda_body_reg_bin(FILE *f, Body *b, long reg, long first, double incrt)
{
    double rreg = (reg + first) * incrt;
    
    fwrite(&rreg, sizeof(double), 1, f);
    fwrite(&(b->reg[reg].pos), sizeof(Triplet), 1, f);
    fwrite(&(b->reg[reg].v), sizeof(Triplet), 1, f);
    fwrite(&(b->reg[reg].hillr), sizeof(double), 1, f);
    fwrite(&(b->reg[reg].distm), sizeof(double), 1, f);
}

int guarda_datos_astro(System *s, int nbody, long ncycle)
{
    FILE *ft, *fb;
    char fname[101];
    long reg, first;
    double incrt;
    Body *b;
  
    b = &(s->astro[nbody]);
    first = ncycle * s->nregs;
    incrt = s->stepdt * s->nsave;

    if (s->pfsal.bdataformat != 'B') {
        snprintf(fname, 100, "%s/%s.%04ld.dat", s->pfsal.datadir, b->nombre, ncycle);
        ft = fopen(fname, "w");
        if (ft == NULL) {
            fprintf(stderr,"Error al crear %s\n", fname);
            return BAD_RETURN;
        }
        for(reg=0; reg<s->nregs; reg++) {
            guarda_body_reg_txt(ft, b, reg, first, incrt);
        }
        fclose(ft);
    }
  
    if (s->pfsal.bdataformat != 'T') {
        snprintf(fname, 100, "%s/%s.%04ld.bin", s->pfsal.datadir, b->nombre, ncycle);
        fb = fopen(fname, "wb");
        if (fb == NULL) {
            fprintf(stderr,"Error al crear %s\n", fname);
            return BAD_RETURN;
        }
        for(reg=0; reg<s->nregs; reg++) {
            guarda_body_reg_bin(fb, b, reg, first, incrt);
        }
        fclose(fb);
    }

    return GOOD_RETURN;
}

int guarda_datos_globales(System *s, long ncycle)
{
    FILE *f;
    char fname[101];
    long i, first;
    double incrt;

    snprintf(fname, 100, "%s/dglobal.%04ld.dat", s->pfsal.datadir, ncycle);
  
    f = fopen(fname, "w");
    
    if (f == NULL) {
        fprintf(stderr,"Error al crear %s\n", fname);
        return BAD_RETURN;
    }

    first = ncycle * s->nregs;
    incrt = s->stepdt * s->nsave;

    for(i=0; i<s->nregs; i++) {
        fprintf(f, "%.0f %.15e %.15e", ((i+first)*incrt),
                s->rege[i].e, s->rege[i].diffe);
        if (s->mre_active) {
            fprintf(f, " %.15e %.15e %.15e %.15e",
                    s->rege[i].elost, s->rege[i].diffelost, s->rege[i].ma0, s->rege[i].ra0);
        }
        if (s->tideson > 0.0) {
            fprintf(f, " %.15e %.15e %.15e %.15e %.15e %.15e",
                    s->rege[i].rt.k0, s->rege[i].rt.tl0, s->rege[i].rt.fac0,
                    s->rege[i].rt.facp, s->rege[i].rt.prefac0, s->rege[i].rt.prefacp);
        }
        fprintf(f, " %.15e %.15e %.15e", s->rege[i].L.x, s->rege[i].L.y, s->rege[i].L.z);
        fprintf(f, "\n");
    }
    fclose(f);
  
    return GOOD_RETURN;
}

int guarda_datos_mind(System *s, long ncycle)
{
    FILE *f;
    char fname[101];
    int i, j ,ind;

    snprintf(fname, 100, "%s/mindist.%04ld.dat", s->pfsal.datadir, ncycle);
  
    f = fopen(fname, "w");
    
    if (f == NULL) {
        fprintf(stderr,"Error al crear %s\n", fname);
        return BAD_RETURN;
    }

    ind = 0;
    for (i=0; i<s->nbodies-1; i++) {
        for (j=i+1; j<s->nbodies; j++) {
            fprintf(f, "%d %d %.15e %.15e %.15e\n", i, j,
                    s->rdist[ind].tmin, s->rdist[ind].dmin, s->rdist[ind].dlim);
            ind++;
        }
    }
  
    fclose(f);
  
    return GOOD_RETURN;
}
