/* SIMulador Sistema PLAnetario, M.Alvarez 2018
 * Trabajo para el TFM
 * del Master de Astrofísica de la UAM
 * 
 * Copyright (c) 2018 Mariano Alvarez Fernandez (malfer@telefonica.net)
 *
 * This file is part of simspla
 *
 * simspla is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * simspla is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with rebound.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdio.h>
#include <stdlib.h>
//#include "mrevol.h"
#include "simspla.h"

MREvolution *mre_new(long nranges)
{
    MREvolution *mre;
    long mem;
  
    mre = (MREvolution *)malloc(sizeof(MREvolution));
    if (mre == NULL) {
        fprintf(stderr, "Error de memoria al crear MREvolution\n");
        return NULL;
    }

    mre->range = (MRERange *)malloc(nranges * sizeof(MRERange));
    if (mre->range == NULL) {
        free(mre);
        fprintf(stderr, "Error de memoria al crear rangos de MEvolution\n");
        return NULL;
    }

    mre->nranges = nranges;
  
    mem = nranges * sizeof(MRERange);
    printflog("Creado MREvolution, memoria estimada %ld\n", mem);

    return mre;
}

MREvolution *mre_load(char *fname)
{
    FILE *f;
    char buf[301];
    MREvolution *mre;
    long nreg, nil1, nil2;
    double rnil1, rnil2, rnil3, rnil4, rnil5;
    double rant, mant, tant;
    int primera_linea;
  
    f = fopen(fname, "r");
    if (f == NULL) {
        fprintf(stderr, "Error al abrir %s\n", fname);
        return NULL;
    }

    nreg = 0;
    while (fgets(buf, 300, f) != NULL) {
        if (buf[0] == '#') continue;
        nreg++;
    }
  
    mre = mre_new(nreg);
    if (mre == NULL) return NULL;
  
    rewind(f);
    nreg = 0;
    rant = 1.0;
    mant = 1.0;
    tant = 0.0;
    primera_linea = 1;
    
    while (fgets(buf, 300, f) != NULL) {
        if (buf[0] == '#') continue;
        if (nreg < mre->nranges) {
            sscanf(buf,"%ld %ld %le %le %le %le %le %le %le %le %le %le",
                   &nil1, &nil2, &rnil1, &rnil2, &(mre->range[nreg].r), &rnil3,
                   &rnil4, &rnil5, &(mre->range[nreg].mlos), &(mre->range[nreg].m),
                   &(mre->range[nreg].dt), &(mre->range[nreg].t));
                if (primera_linea) {
                    rant = mre->range[nreg].r;
                    mant = mre->range[nreg].m;
                    primera_linea = 0;
                }
            // recalculo mlos y dt para ser mas preciso
            mre->range[nreg].dt = mre->range[nreg].t - tant;
            mre->range[nreg].mlos = (mant - mre->range[nreg].m) / mre->range[nreg].dt;
            mre->range[nreg].rlos = (rant - mre->range[nreg].r) / mre->range[nreg].dt;
            rant = mre->range[nreg].r;
            mant = mre->range[nreg].m;
            tant = mre->range[nreg].t;
            nreg++;
        }
    }
  
    return mre;
}

int mre_setinit(MREvolution *mre, double t, double tsim, double utsim, double msim, double rsim)
/* fija el momento de incio de la simulación en t (años), informa de tsim, utsim
 * y de la masa y radio en ese momento en unidades de la simulación */
{
    long i, init_range = -1;
    double incrt;
  
    for (i=0; i<mre->nranges; i++) {
        if (mre->range[i].t > t) {
            init_range = i;
            break;
        }
    }
  
    if (init_range < 0) {
        fprintf(stderr, "Error en mre_setinit\n");
        return BAD_RETURN;
    }
  
    incrt = mre->range[init_range].t - t;
    mre->init_range = init_range;
    mre->init_msim = msim / (mre->range[init_range].m +
                   (incrt * mre->range[init_range].mlos));
    mre->init_rsim = rsim / (mre->range[init_range].r +
                   (incrt * mre->range[init_range].rlos));
    mre->init_t = t;
    mre->init_tsim = tsim;
    mre->utsim = utsim;
    mre->act_range = init_range;
  
    return GOOD_RETURN;
}

void mre_getmrsim(MREvolution *mre, double tsim, double *msim, double *rsim)
{
    double t, incrt;
    long reg;
  
    t = mre->init_t + ((tsim - mre->init_tsim) * mre->utsim);
  
    while ((mre->act_range < mre->nranges) && (t > mre->range[mre->act_range].t)) {
        mre->act_range += 1;
    }

    reg = mre->act_range;

    incrt = mre->range[reg].t - t;
    //*rsim = mre->range[reg].r * mre->init_rsim;
    *rsim = (mre->range[reg].r + (incrt * mre->range[reg].rlos)) * mre->init_rsim;
    *msim = (mre->range[reg].m + (incrt * mre->range[reg].mlos)) * mre->init_msim;
}
