/* SIMulador Sistema PLAnetario, M.Alvarez 2018
 * Trabajo para el TFM
 * del Master de Astrofísica de la UAM
 * 
 * Copyright (c) 2018 Mariano Alvarez Fernandez (malfer@telefonica.net)
 *
 * This file is part of simspla
 *
 * simspla is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * simspla is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with rebound.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include "simspla.h"
#include "rebound.h"
#include "myrtools.h"

#define FLOG "simspla.log"

double G_tmax, G_tini, G_tcheck, G_emr_tcheck, G_elost;

/* Actualiza la masa y radio del cuerpo principal,
 * calcula encuentros cercanos y vaporiza planetas
 */
void post_timestep_modifications(struct reb_simulation* r)
{
    System *s;
    double masa, radio;
    double eb, ea;
    int i, j, ind;
    double ced;

    s = (System *)(r->extras);
    
    calcula_dist(r);

    // Calcula si el astro central engulle planetas
    for (int j=1;j<s->nbodies;j++){
        if (s->astro[j].estado != 0) continue;
        if (r->particles[0].r > s->rdist[j-1].d) {
            eb = reb_tools_energy(r);
            vaporiza_planeta(r, j);
            ea = reb_tools_energy(r);
            printflog("\nVaporizado planeta %d en t=%f\n", j, r->t);
            printflog("Eam:%.15e Eaf:%.15e DiffE:%.15e\n", eb, ea, eb-ea);
            G_elost += eb - ea;
        }
    }
    
    // Calcula si hay encuentros cercanos
    ind = s->nbodies - 1;
    for (i=1; i<s->nbodies-1; i++) {
        if (s->astro[i].estado == 0) {
            for (j=i+1; j<s->nbodies; j++) {
                if (s->astro[j].estado == 0) {
                    ced = (s->astro[i].hillr + s->astro[j].hillr) * s->mul_rh;
                    if (ced > s->rdist[ind].d) {
                        if (s->rdist[ind].aec == 0) {
                            printflog("\nAviso, encuentro cercano entre planetas %d y %d\n", i, j);
                            printflog("t=%f Limite:%e Distancia:%e\n", r->t, ced, s->rdist[ind].d);
                            s->rdist[ind].aec = 1;
                        }
                    } else {
                        s->rdist[ind].aec = 0;
                    }
                }
                ind++;
            }
        } else {
            ind += s->nbodies - 1 - i;
        }
    }
    
    // Acumula para calculo distancia media
    for (i=1; i<s->nbodies; i++) {
        if (s->astro[i].estado == 0) {
            s->astro[i].acudistm += s->rdist[i-1].d;
        }
    }

    // Evoluciona masa y radio del astro central
    //if ((s->mre_active) && (floor(r->t/G_emr_tcheck)!=floor((r->t-r->dt)/G_emr_tcheck))) {
    if (s->mre_active) {
        eb = my_reb_tools_energy_a0(r);
        mre_getmrsim(s->mre, r->t, &masa, &radio);
        r->particles[0].m = masa + s->m0_acu;
        r->particles[0].r = radio;
        ea = my_reb_tools_energy_a0(r);
        G_elost += eb - ea;
    }
}

/* Cada nsave pasos muestra estado, calcula radios de Hill y guarda los datos
 */
void heartbeat(struct reb_simulation* r)
{
    System *s;
    double e;
    long intv;
    int i, indrb;
    struct reb_vec3d L;

    //if (reb_output_check(r, G_tcheck)){
    if (floor(r->t/G_tcheck)!=floor((r->t-r->dt)/G_tcheck)) {
        s = (System *)(r->extras);
        if ((s->showpro > 0) && (++(s->countsp) >= s->showpro)) {
          my_reb_output_timing(r, G_tini, G_tmax);
          s->countsp = 0;
        }
        reb_integrator_synchronize(r);
        calcula_hillr(r);
        intv = (r->t - G_tini) / G_tcheck;
        for (i=0; i<s->nbodies; i++) {
            if (s->astro[i].estado == 0) {
                indrb = s->astro[i].indrb;
                s->astro[i].reg[intv].pos.x = r->particles[indrb].x;
                s->astro[i].reg[intv].pos.y = r->particles[indrb].y;
                s->astro[i].reg[intv].pos.z = r->particles[indrb].z;
                s->astro[i].reg[intv].v.x = r->particles[indrb].vx;
                s->astro[i].reg[intv].v.y = r->particles[indrb].vy;
                s->astro[i].reg[intv].v.z = r->particles[indrb].vz;
                s->astro[i].reg[intv].hillr = s->astro[i].hillr;
                if (i>0) {
                    s->astro[i].reg[intv].distm = s->astro[i].acudistm / s->nsave;
                    s->astro[i].acudistm = 0;
                }
            } else {
                s->astro[i].reg[intv].pos.x = 0.0;
                s->astro[i].reg[intv].pos.y = 0.0;
                s->astro[i].reg[intv].pos.z = 0.0;
                s->astro[i].reg[intv].v.x = 0.0;
                s->astro[i].reg[intv].v.y = 0.0;
                s->astro[i].reg[intv].v.z = 0.0;
                s->astro[i].reg[intv].hillr = 0.0;
                s->astro[i].reg[intv].distm = 0.0;
            }
        }
        e = reb_tools_energy(r);
        s->rege[intv].e = e;
        s->rege[intv].diffe = fabs((e-s->eini)/s->eini);
        if (s->mre_active) {
            s->rege[intv].elost = G_elost;
            s->rege[intv].diffelost = fabs((e+G_elost-s->eini)/s->eini);
            s->rege[intv].ma0 = r->particles[0].m;
            s->rege[intv].ra0 = r->particles[0].r;
            s->rege[intv].rt = s->rt;
        }
        L = reb_tools_angular_momentum(r);
        s->rege[intv].L.x = L.x;
        s->rege[intv].L.y = L.y;
        s->rege[intv].L.z = L.z;
    }
}

/* Prepara r para la integración
 */
void setup_simulation(System *s, struct reb_simulation* r)
{
    double masa, radio;
    int i, indrb;

    r->t = s->t0;
    r->dt = s->stepdt;
    r->G = s->G;
    
    if (s->integrator == INTG_DKD) {
        r->integrator = REB_INTEGRATOR_LEAPFROG;
    } else {
        r->ri_whfast.safe_mode 	= 1;        // Turn on safe mode
        r->ri_whfast.corrector 	= 0;        // No symplectic corrector
        r->ri_whfast.coordinates = REB_WHFAST_COORDINATES_DEMOCRATICHELIOCENTRIC;
        r->integrator = REB_INTEGRATOR_WHFAST;
    }
    
    r->heartbeat = heartbeat;
    r->post_timestep_modifications = post_timestep_modifications;
    r->additional_forces = additional_forces_tides5;
    r->force_is_velocity_dependent = 1;
    r->exact_finish_time = 1;           // Finish exactly at G_tmax in reb_integrate(). Default is already 1.

    indrb = 0;
    for (i=0; i<s->nbodies; i++){
        if (s->astro[i].estado != 0) continue;
        struct reb_particle p = {0};
        p.x = s->astro[i].pos.x;
        p.y = s->astro[i].pos.y;
        p.z = s->astro[i].pos.z;
        p.vx = s->astro[i].v.x;
        p.vy = s->astro[i].v.y;
        p.vz = s->astro[i].v.z;
        p.m  = s->astro[i].masa;
        p.r  = s->astro[i].radio;
        reb_add(r, p);
        s->astro[i].indrb = indrb;
        indrb++;
    }
    
    if (s->mre_active && s->nfcycle > 0) {
        mre_getmrsim(s->mre, r->t, &masa, &radio);
        r->particles[0].m = masa;
        r->particles[0].r = radio;
        //printf("EMR %.17e,%.17e\n", masa, radio);
    }

    if (s->eini == 0) s->eini = reb_tools_energy(r);

    r->extras = (void *)s;
}

/* Ejecuta un ciclo
 */
void run_cycle(System *s, struct reb_simulation* r, long ncycle)
{
    int i, result;
    struct timeval tim1, tim2;
    long temp_m, temp_s;
    
    gettimeofday(&tim1, NULL);
    printflog("\nCalculando ciclo %ld de %ld, t=%f\n", ncycle, s->ncycles, r->t);
    
    if (s->showpro > 0 && !s->pfsal.showlcon)
        printf("\nCalculando ciclo %ld de %ld\n", ncycle, s->ncycles);

    G_tini = r->t;
    G_tmax = r->dt * s->nsteps * (ncycle + 1);
    G_tcheck = r->dt * s->nsave;
    G_emr_tcheck = s->mre_active ? r->dt * s->mre_evstep : 0.0;

    for (i=0; i<s->nreg_dist; i++) {
        s->rdist[i].d = 0;
        s->rdist[i].dmin = 1e99;
        s->rdist[i].tmin = 0;
        s->rdist[i].dlim = 0;
        s->rdist[i].aec = 0;
    }
    calcula_hillr(r);
    calcula_dist(r);
    
    s->countsp = 0;
    
    result = reb_integrate(r, G_tmax);
    if (result == REB_EXIT_SIGINT) {
        putslog("\nSimulación abortada\n");
        exit(1);
    }
  
    gettimeofday(&tim2, NULL);
    temp_m = floor((tim2.tv_sec - tim1.tv_sec) / 60.0);
    temp_s = tim2.tv_sec - tim1.tv_sec - temp_m * 60;
    printflog("\nGuardando ficheros ciclo %ld, tsys=%f, cpu=%ld\'%ld\"\n",
              ncycle, r->t, temp_m, temp_s);
    flushlog();

    if (s->pfsal.genbdata) {
        for (i=0; i<s->nbodies; i++) guarda_datos_astro(s, i, ncycle);
    }
    if (s->pfsal.gengdata) guarda_datos_globales(s, ncycle);
    if (s->pfsal.genmindist) guarda_datos_mind(s, ncycle);
}

/* Programa principal
 */
int main(int argc, char** argv)
{
    System *s;
    struct reb_simulation* r;
    long ncycle;
    
    if (argc < 2) {
        fprintf(stderr, "Ejecutar con ./simspla fichero.def\n");
        return 1;
    }

    s = genera_system_from_filedef(argv[1]);
    if (s == NULL) {
        fprintf(stderr, "Error al generar el Sistema\n");
        return 1;
    }
    
    openlog(s->pfsal.logfile, s->pfsal.showlcon);
    printflog("%s\n\n", VERSION);

    r = reb_create_simulation();
    if (r == NULL) {
        fprintf(stderr, "Error al crear reb_simulation\n");
        return 1;
    }

    if (s->mre_fname != NULL) {
        s->mre = mre_load(s->mre_fname);
        if (s->mre == NULL) return 1;
        if (mre_setinit(s->mre, s->mre_t0, 0.0, s->mre_utsim, s->astro[0].masa,
                        s->astro[0].radio) != GOOD_RETURN) return 1;
        s->mre_active = 1;
        G_elost = s->elost_acu;
    }
    
    setup_simulation(s, r);
    
    //reb_move_to_com(r); // no es necesario y cambia todas las coordenadas
    
    ncycle = s->nfcycle;
    while (1) {
        if (ncycle >= s->ncycles) break;
        run_cycle(s, r, ncycle);
        ncycle++;
        s->elost_acu = G_elost;
        if (s->pfsal.gendeffiles) guarda_system_to_filedef(s, ncycle);
    }

  return 0;
}
