/* SIMulador Sistema PLAnetario, M.Alvarez 2018
 * Trabajo para el TFM
 * del Master de Astrofísica de la UAM
 * 
 * Copyright (c) 2018 Mariano Alvarez Fernandez (malfer@telefonica.net)
 *
 * This file is part of simspla
 *
 * simspla is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * simspla is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with rebound.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include "simspla.h"
#include "rebound.h"
#include "myrtools.h"

/* 
 */
void additional_forces_tides1(struct reb_simulation* r)
{
    System *s;
    struct reb_particle *p0, *pi;
    int i;
    double dx, dy, dz, fac0, mratio, fac, dr2, prefac;
    double k10 = 0.03;
    double k1p = 0.3;

    s = (System *)(r->extras);

    p0 = &(r->particles[0]);
    for (i=1; i<s->nbodies-1; i++) {
        if (s->astro[i].estado == 0) {
            pi = &(r->particles[s->astro[i].indrb]);
            fac0 = k10 * p0->r * p0->r * p0->r * p0->r * p0->r;
            mratio = pi->m / p0->m; // ¿comprobar si menor que DBL_MIN?
            fac = fac0 * mratio;
            fac += k1p * pi->r * pi->r * pi->r * pi->r * pi->r / mratio;
            dx = pi->x - p0->x;
            dy = pi->y - p0->y;
            dz = pi->z - p0->z;
            dr2 = dx*dx + dy*dy + dz*dz;
            prefac = -3 * r->G * (p0->m + pi->m) / (dr2 * dr2 *dr2 * dr2) * fac;
            pi->ax += prefac * dx;
            pi->ay += prefac * dy;
            pi->az += prefac * dz;
            p0->ax -= mratio * prefac * dx;
            p0->ay -= mratio * prefac * dy;
            p0->az -= mratio * prefac * dz;
        }
    }
}

/* 
 */
void additional_forces_tides2(struct reb_simulation* r)
{
    System *s;
    struct reb_particle *p0, *pi;
    int i;
    double dx, dy, dz, dr2;
    double fac0, facp, prefac, prefac0, prefacp;
    double k10 = 0.03;
    double k1p = 0.3;

    s = (System *)(r->extras);

    p0 = &(r->particles[0]);
    for (i=1; i<s->nbodies-1; i++) {
        if (s->astro[i].estado == 0) {
            pi = &(r->particles[s->astro[i].indrb]);
            dx = pi->x - p0->x;
            dy = pi->y - p0->y;
            dz = pi->z - p0->z;
            dr2 = dx*dx + dy*dy + dz*dz;
            fac0 = pi->m * pi->m * k10 * p0->r * p0->r * p0->r * p0->r * p0->r;
            facp = p0->m * p0->m * k1p * pi->r * pi->r * pi->r * pi->r * pi->r;
            prefac = -3 * r->G * (fac0 + facp) / (dr2 * dr2 *dr2 * dr2);
            prefac0 = prefac / p0->m;
            prefacp = prefac / pi->m;
            p0->ax -= prefac0 * dx;
            p0->ay -= prefac0 * dy;
            p0->az -= prefac0 * dz;
            pi->ax += prefacp * dx;
            pi->ay += prefacp * dy;
            pi->az += prefacp * dz;
        }
    }
}


/* 
 */
void additional_forces_tides3(struct reb_simulation* r)
{
    System *s;
    struct reb_particle *p0, *pi;
    int i;
    double dx, dy, dz, dr2;
    double dvx, dvy, dvz, dvr;
    double fac0, facp, prefac, prefac0, prefacp;
    double k0, kp, tl0, tlp, r05, rp5;
    double tideson2;

    s = (System *)(r->extras);
    if (s->tideson <= 0.0) return; // no se calculan tides
    
    tideson2 = s->tideson * s->tideson;

    p0 = &(r->particles[0]);
    r05 = p0->r * p0->r * p0->r * p0->r * p0->r;
    k0 = s->astro[0].k;
    tl0 = s->astro[0].tl;
    if (tl0 == 0.0) tl0 = (3 * r05 * s->astro[0].sig) / (2 * r->G);
        
    for (i=1; i<s->nbodies; i++) {
        if (s->astro[i].estado == 0) {
            pi = &(r->particles[s->astro[i].indrb]);
            dx = pi->x - p0->x;
            dy = pi->y - p0->y;
            dz = pi->z - p0->z;
            dr2 = dx*dx + dy*dy + dz*dz;
            if (dr2 >= tideson2) continue; // radio mayor del máximo configurado
            dvx = pi->vx - p0->vx;
            dvy = pi->vy - p0->vy;
            dvz = pi->vz - p0->vz;
            dvr = dx*dvx + dy*dvy + dz*dvz;
            
            rp5 = pi->r * pi->r * pi->r * pi->r * pi->r;
            kp = s->astro[i].k;
            tlp = s->astro[i].tl;
            if (tlp == 0.0) tlp = (3 * rp5 * s->astro[i].sig) / (2 * r->G);

            fac0 = pi->m * pi->m * k0 * r05 * (1 + (3 * (+dvr) * tl0 / dr2));
            facp = p0->m * p0->m * kp * rp5 * (1 + (3 * (+dvr) * tlp / dr2));
            prefac = -3 * r->G * (fac0 + facp) / (dr2 * dr2 *dr2 * dr2);
            prefac0 = prefac / p0->m;
            prefacp = prefac / pi->m;
            p0->ax -= prefac0 * dx;
            p0->ay -= prefac0 * dy;
            p0->az -= prefac0 * dz;
            pi->ax += prefacp * dx;
            pi->ay += prefacp * dy;
            pi->az += prefacp * dz;
        }
    }
}

/* 
 */
void additional_forces_tides4(struct reb_simulation* r)
{
    System *s;
    struct reb_particle *p0, *pi;
    int i;
    double dx, dy, dz, dr2;
    double dvx, dvy, dvz, dvr;
    double fac0, facp, prefac, prefac0, prefacp;
    double k0, kp, tl0, tlp, r05, rp5;
    double delta, tideson2;

    s = (System *)(r->extras);
    if (s->tideson <= 0.0) return; // no se calculan tides
    
    tideson2 = s->tideson * s->tideson;

    p0 = &(r->particles[0]);
    r05 = p0->r * p0->r * p0->r * p0->r * p0->r;
    k0 = s->astro[0].k;
    tl0 = s->astro[0].tl;
    if (s->astro[0].sig > 0.0) {
        delta = sqrt(s->astro[0].presig * r05);
        k0 = k0 * delta;
        tl0 = tl0 * delta;
    }

    for (i=1; i<s->nbodies; i++) {
        if (s->astro[i].estado == 0) {
            pi = &(r->particles[s->astro[i].indrb]);
            dx = pi->x - p0->x;
            dy = pi->y - p0->y;
            dz = pi->z - p0->z;
            dr2 = dx*dx + dy*dy + dz*dz;
            if (dr2 >= tideson2) continue; // radio mayor del máximo configurado
            dvx = pi->vx - p0->vx;
            dvy = pi->vy - p0->vy;
            dvz = pi->vz - p0->vz;
            dvr = dx*dvx + dy*dvy + dz*dvz;
            
            rp5 = pi->r * pi->r * pi->r * pi->r * pi->r;
            kp = s->astro[i].k;
            tlp = s->astro[i].tl;
            //if (tlp == 0.0) tlp = (3 * rp5 * s->astro[i].sig) / (2 * r->G);

            fac0 = pi->m * pi->m * k0 * r05 * (1 + (3 * (+dvr) * tl0 / dr2));
            facp = p0->m * p0->m * kp * rp5 * (1 + (3 * (+dvr) * tlp / dr2));
            prefac = -3 * r->G * (fac0 + facp) / (dr2 * dr2 *dr2 * dr2);
            s->astro[0].hillr = prefac; // buscando bug
            prefac0 = prefac / p0->m;
            prefacp = prefac / pi->m;
            p0->ax -= prefac0 * dx;
            p0->ay -= prefac0 * dy;
            p0->az -= prefac0 * dz;
            pi->ax += prefacp * dx;
            pi->ay += prefacp * dy;
            pi->az += prefacp * dz;
        }
    }
}

/* 
 */
void additional_forces_tides5(struct reb_simulation* r)
{
    System *s;
    struct reb_particle *p0, *pi;
    int i;
    double dx, dy, dz, dr2;
    double dvx, dvy, dvz, dvr;
    double fac0, facp, prefac, prefac0, prefacp;
    double k0, kp, tl0, tlp, r05, rp5;
    double deltatl, tideson2;

    s = (System *)(r->extras);
    if (s->tideson <= 0.0) return; // no se calculan tides
    if (s->astro[0].tmode == TMODE_NOTIDES) return;
    
    tideson2 = s->tideson * s->tideson;

    p0 = &(r->particles[0]);
    r05 = p0->r * p0->r * p0->r * p0->r * p0->r;
    k0 = s->astro[0].k;
    tl0 = s->astro[0].tl;
    if (s->astro[0].tmode == TMODE_EVOL) {
        deltatl = sqrt(s->astro[0].presig * r05);
        k0 = k0 * s->astro[0].fr * deltatl;
        tl0 = tl0 * deltatl;
    }

    for (i=1; i<s->nbodies; i++) {
        if (s->astro[i].estado != 0) continue;
        if (s->astro[i].tmode == TMODE_NOTIDES) continue;

        pi = &(r->particles[s->astro[i].indrb]);
        dx = pi->x - p0->x;
        dy = pi->y - p0->y;
        dz = pi->z - p0->z;
        dr2 = dx*dx + dy*dy + dz*dz;
        if (dr2 >= tideson2) continue; // radio mayor del máximo configurado
        dvx = pi->vx - p0->vx;
        dvy = pi->vy - p0->vy;
        dvz = pi->vz - p0->vz;
        dvr = dx*dvx + dy*dvy + dz*dvz;
            
        rp5 = pi->r * pi->r * pi->r * pi->r * pi->r;
        kp = s->astro[i].k;
        tlp = s->astro[i].tl;
        //falta posibilidad de TMODE_EVOL en planetas
        
        fac0 = pi->m * pi->m * k0 * r05 * (1 + (3 * (+dvr) * tl0 / dr2));
        facp = p0->m * p0->m * kp * rp5 * (1 + (3 * (+dvr) * tlp / dr2));
        prefac = -3 * r->G * (fac0 + facp) / (dr2 * dr2 *dr2 * dr2);
        prefac0 = prefac / p0->m;
        prefacp = prefac / pi->m;
        p0->ax -= prefac0 * dx;
        p0->ay -= prefac0 * dy;
        p0->az -= prefac0 * dz;
        pi->ax += prefacp * dx;
        pi->ay += prefacp * dy;
        pi->az += prefacp * dz;
        if (i==1) {
            s->rt.k0 = k0;
            s->rt.tl0 = tl0;
            s->rt.fac0 = fac0;
            s->rt.facp = facp;
            s->rt.prefac0 = prefac0;
            s->rt.prefacp = prefacp;
        }
    }
}
